function getfzp, pbx, pb $
               , pbergs=pbergs, vegafile=vegafile, vegamags=vegamags $
               , outvf=outvf, outvm=outvm, outvlameff=outvlameff $
               , outmag=outmag, closest=closest
;
; getfzp.pro -- returns filter zeropoint by integrating a Vega spectrum
;               
; INPUTS
;      pbx: wavelength points for passband  (Angstroms)
;       pb: passband (photon or energy sensitivity)
;
; OPTIONAL INPUTS
;   pbergs: set /pbergs if passband response is energy sensitivity
;           (it's assumed to be photons by default)
;
; vegafile: Vega f_lambda spectrum filename to use 
;           default: alpha_lyr_stis_003.dat (in $MLCS2K2_BASEDIR/aux/standards)
;
; vegamags: set of Vega standard magnitudes to use, from
;           $MLCS2K2_BASEDIR/aux/standards/VEGA.mags.dat
;           default: Bohlin06
;
; closest:  set this keyword to use the closest entry in the magnitude file
;           rather than linearly interpolating (use this when you want
;           the passband to give exactly a certain Vega magnitude)
;
; OPTIONAL OUTPUS
;    outvf: Vega file used
;    outvm: Vega standard magnitudes used
;    outvlameff: Vega weighted effective wavelength
;    outmag: Vega magnitude in passband
;

if n_elements(vegafile) eq 0 then vegafile = 'alpha_lyr_stis_003.dat'
if n_elements(vegamags) eq 0 then vegamags = 'Bohlin06'
closest = keyword_set(closest)

outvf = vegafile
outvm = vegamags

bdir = getenv('MLCS2K2_BASEDIR')
if bdir eq '' then message,' environment variable MLCS2K2_BASEDIR not set'

vf = bdir + '/aux/standards/' + vegafile
readspec,vf,vegalam,vegaflm

vm = bdir + '/aux/standards/VEGA.mags.dat'
method = ''
readit,vm,u0,b0,v0,r0,i0,j0,h0,k0,method

eff = where(method eq 'vlameff',neff)
if neff ne 1 then message,' invalid file: ' + vm

meth = where(method eq vegamags,nmeth)
if nmeth eq 0 then message,vegamags + ' method not found in ' + vm
if nmeth gt 1 then message,vegamags + ' method duplicated in ' + vm

vleff = [u0[eff],b0[eff],v0[eff],r0[eff],i0[eff],j0[eff],h0[eff],k0[eff]]
zeros = [u0[meth],b0[meth],v0[meth],r0[meth],i0[meth],j0[meth],h0[meth],k0[meth]]
nz = n_elements(zeros)

; calculate vega-weighted effective wavelength for the requested filter
pvleff = synflux2(vegalam,vegalam*vegaflm,pbx,pb,pbergs=pbergs) / $
         synflux2(vegalam,vegaflm,pbx,pb,pbergs=pbergs)
outvlameff = pvleff

if closest then begin
    junk = min(abs(pvleff-vleff),vlind)
    mvega = zeros[vlind]
    message,'effective wavelength: ' + string(pvleff,format='(f0.2)') + $
      '   closest match: ' + string(vleff[vlind],format='(f0.2)'),/info
endif else begin
    ; interpolate vega magnitudes to get the desired zeropoint
    ;   no extrapolation, we just set to U or K as needed
    if pvleff lt min(vleff) then mvega = zeros[0] $
    else if pvleff gt max(vleff) then mvega = zeros[nz-1] $
    else mvega = interpol(zeros,vleff,pvleff)
    message,'effective wavelength: ' + string(pvleff,format='(f0.2)') + $
      '   interpolating.',/info
endelse
outmag = mvega

; finally we return the zeropoint, such that 
; synphot2(vegalam,vegaflm,pbx,pb,zp) will return mvega
return, mvega - synphot2(vegalam,vegaflm,pbx,pb,0.0d,pbergs=pbergs)

end
