function getgenfzp, pbx, pb $
               , pbergs=pbergs, modelspec=modelspec, modelmags=modelmags $
               , modelfile=modelfile, outvs=outvs $
               , outvf=outvf, outvm=outvm, outvlameff=outvlameff $
               , outmag=outmag, closest=closest
;
; getgenfzp.pro -- returns filter zeropoint by integrating a model spectrum
;               
; INPUTS
;      pbx: wavelength points for passband  (Angstroms)
;       pb: passband (photon or energy sensitivity)
;
; OPTIONAL INPUTS
;   pbergs: set /pbergs if passband response is energy sensitivity
;           (it's assumed to be photons by default)
;
; modelspec: Model f_lambda spectrum filename to use 
;           default: AB.flm (in $MLCS2K2_BASEDIR/aux/standards)
;
; modelfile: name of file with model magnitudes
;           default: AB.mags.dat (in $MLCS2K2_BASEDIR/aux/standards)
;
; modelmags: set of Model standard magnitudes to use, from modelfile
;           default: ZeroByDefn
;
; closest:  set this keyword to use the closest entry in the magnitude file
;           rather than linearly interpolating (use this when you want
;           the passband to give exactly a certain model magnitude)
;
; OPTIONAL OUTPUS
;    outvs: Model spectrum used
;    outvf: Model file used
;    outvm: Model standard magnitudes used
;    outvlameff: Model weighted effective wavelength
;    outmag: Model magnitude in passband
;

if n_elements(modelspec) eq 0 then modelspec = 'AB.flm'
if n_elements(modelfile) eq 0 then modelfile = 'AB.mags.dat'
if n_elements(modelmags) eq 0 then modelmags = 'ZeroByDefn'
closest = keyword_set(closest)

outvs = modelspec
outvf = modelfile
outvm = modelmags

bdir = getenv('MLCS2K2_BASEDIR')
if bdir eq '' then message,' environment variable MLCS2K2_BASEDIR not set'

vf = bdir + '/aux/standards/' + modelspec
readspec,vf,modellam,modelflm

vm = bdir + '/aux/standards/' + modelfile
method = ''
readit,vm,u0,b0,v0,r0,i0,j0,h0,k0,method

eff = where(method eq 'lameff',neff)
if neff ne 1 then message,' invalid file: ' + vm

meth = where(method eq modelmags,nmeth)
if nmeth eq 0 then message,modelmags + ' method not found in ' + vm
if nmeth gt 1 then message,modelmags + ' method duplicated in ' + vm

vleff = [u0[eff],b0[eff],v0[eff],r0[eff],i0[eff],j0[eff],h0[eff],k0[eff]]
zeros = [u0[meth],b0[meth],v0[meth],r0[meth],i0[meth],j0[meth],h0[meth],k0[meth]]
nz = n_elements(zeros)

; calculate model-weighted effective wavelength for the requested filter
pvleff = synflux2(modellam,modellam*modelflm,pbx,pb,pbergs=pbergs) / $
         synflux2(modellam,modelflm,pbx,pb,pbergs=pbergs)
outvlameff = pvleff

if closest then begin
    junk = min(abs(pvleff-vleff),vlind)
    mmodel = zeros[vlind]
    message,'effective wavelength: ' + string(pvleff,format='(f0.2)') + $
      '   closest match: ' + string(vleff[vlind],format='(f0.2)'),/info
endif else begin
    ; interpolate model magnitudes to get the desired zeropoint
    ;   no extrapolation, we just set to U or K as needed
    if pvleff lt min(vleff) then mmodel = zeros[0] $
    else if pvleff gt max(vleff) then mmodel = zeros[nz-1] $
    else mmodel = interpol(zeros,vleff,pvleff)
    message,'effective wavelength: ' + string(pvleff,format='(f0.2)') + $
      '   interpolating.',/info
endelse
outmag = mmodel

; finally we return the zeropoint, such that 
; synphot2(modellam,modelflm,pbx,pb,zp) will return mmodel
return, mmodel - synphot2(modellam,modelflm,pbx,pb,0.0d,pbergs=pbergs)

end
