pro mkgenpassband,name,pbx,pb $
              ,pbergs=pbergs, modelspec=modelspec, modelmags=modelmags $
              ,modelfile=modelfile,closest=closest,force=force, history=history
;
; mkgenpassband.pro -- creates the passband fits file 
;   but using a generic model spectrum (not necessarily Vega)
;       
; Created 2006-Jul-10
; Version 2006-Oct-12
;
; INPUTS
;     name: name of the passband (will create name+'.fits')
;      pbx: wavelength points for passband  (Angstroms)
;       pb: passband response (photon or energy sensitivity)
;
; OPTIONAL INPUTS
;   pbergs: set /pbergs if passband response is energy sensitivity
;           (it's assumed to be photons by default)
;
; modelspec: Model f_lambda spectrum filename to use 
;           default: AB.flm (in $MLCS2K2_BASEDIR/aux/standards)
;
; modelfile: name of file with model magnitudes
;           default: AB.mags.dat (in $MLCS2K2_BASEDIR/aux/standards)
;
; modelmags: set of Model standard magnitudes to use, from
;           $MLCS2K2_BASEDIR/aux/standards/MODEL.mags.dat
;           default: ZeroByDefn
;
; closest:  set this keyword to use the closest entry in the magnitude file
;           rather than linearly interpolating (use this when you want
;           the passband to give exactly a certain model magnitude)
;
;    force: overwrite output file if it exists
;
;  history: a string (or string array) of additional history
;           information to put into the passband fits file. For
;           example, you could add a reference, history='Bessell (1990)'
;           

pbergs = keyword_set(pbergs)
if n_elements(modelspec) eq 0 then modelspec = 'AB.flm'
if n_elements(modelfile) eq 0 then modelfile = 'AB.mags.dat'
if n_elements(modelmags) eq 0 then modelmags = 'ZeroByDefn'
closest = keyword_set(closest)
force = keyword_set(force)
if n_elements(history) gt 0 then addhist = 1 else addhist = 0

if strpos(name,'.') ne -1 then $
  message,'no periods allowed in passband names: ' + name

outfn = name + '.fits'
if ((not force) and (file_test(outfn))) then $
  message,outfn+' already exists! use /force if you really mean it.'

n = n_elements(pbx)
if n_elements(pb) ne n then message,'incompatible arrays: pbx and pb'

if pb[0] ne 0. or pb[n-1] ne 0. then $
  message,'passband probably should be zero at both ends!',/info

print,'creating ' + outfn + ' ...'
passband = replicate({wavelength:0.0d, response:0.0d},n)
passband.wavelength = pbx
passband.response = pb

zp = getgenfzp(pbx,pb,pbergs=pbergs,modelspec=modelspec,modelmags=modelmags, $
               modelfile=modelfile,outvf=vf, outvm=vm, outvs=vs, outmag=m, $
               outvlam=vl,closest=closest)

sxaddpar,hdr,'PBNAME',name,' name of passband'
sxaddpar,hdr,'WAVEUNIT','Angstroms',' all wavelengths in Angstroms'
sxaddpar,hdr,'RESPUNIT',(pbergs ? 'ergs' : 'photons') $
  ,' energy (ergs) or photon sensitivity'
sxaddpar,hdr,'ZEROPT',zp,' passband zeropoint',format='F19.4'
sxaddpar,hdr,'MODLSPEC',vs,' Model spectrum used in aux/standards/'
sxaddpar,hdr,'MODLMFIL',vf,' file containing Model magnitudes'
sxaddpar,hdr,'MODLMAGS',vm,' set of Model magnitudes in MODLMFIL'
sxaddpar,hdr,'MODLWEFF',vl,' Model-weighted effective wavelength',format='F19.4'
sxaddpar,hdr,'MAGMODEL',m,' magnitude of Model in this passband',format='F19.4'
sxaddhist,'Created using mkgenpassband.pro at UT ' + systime(/utc),hdr
sxaddhist,'Created by ' + getenv('USER') + '@' + getenv('HOST'),hdr

if addhist then sxaddhist,history,hdr

mwrfits,passband,outfn,hdr,ascii='D:G20.7',/create,/terminator
print,'done.'

end
