pro mkpassband,name,pbx,pb $
              ,pbergs=pbergs, vegafile=vegafile, vegamags=vegamags $
              ,closest=closest,force=force, history=history
;
; mkpassband.pro -- creates the passband fits file 
;       
; Created 2005-Oct-30
; Version 2006-Oct-12
;
; INPUTS
;     name: name of the passband (will create name+'.fits')
;      pbx: wavelength points for passband  (Angstroms)
;       pb: passband response (photon or energy sensitivity)
;
; OPTIONAL INPUTS
;   pbergs: set /pbergs if passband response is energy sensitivity
;           (it's assumed to be photons by default)
;
; vegafile: Vega f_lambda spectrum filename to use 
;           default: alpha_lyr_stis_003.dat (in $MLCS2K2_BASEDIR/aux/standards)
;
; vegamags: set of Vega standard magnitudes to use, from
;           $MLCS2K2_BASEDIR/aux/standards/VEGA.mags.dat
;           default: Bohlin06
;
; closest:  set this keyword to use the closest entry in the magnitude file
;           rather than linearly interpolating (use this when you want
;           the passband to give exactly a certain Vega magnitude)
;
;    force: overwrite output file if it exists
;
;  history: a string (or string array) of additional history
;           information to put into the passband fits file. For
;           example, you could add a reference, history='Bessell (1990)'
;           

pbergs = keyword_set(pbergs)
if n_elements(vegafile) eq 0 then vegafile = 'alpha_lyr_stis_003.dat'
if n_elements(vegamags) eq 0 then vegamags = 'Bohlin06'
closest = keyword_set(closest)
force = keyword_set(force)
if n_elements(history) gt 0 then addhist = 1 else addhist = 0

if strpos(name,'.') ne -1 then $
  message,'no periods allowed in passband names: ' + name

outfn = name + '.fits'
if ((not force) and (file_test(outfn))) then $
  message,outfn+' already exists! use /force if you really mean it.'

n = n_elements(pbx)
if n_elements(pb) ne n then message,'incompatible arrays: pbx and pb'

if pb[0] ne 0. or pb[n-1] ne 0. then $
  message,'passband probably should be zero at both ends!',/info

print,'creating ' + outfn + ' ...'
passband = replicate({wavelength:0.0d, response:0.0d},n)
passband.wavelength = pbx
passband.response = pb

zp = getfzp(pbx,pb,pbergs=pbergs,vegafile=vegafile,vegamags=vegamags $
            ,outvf=vf,outvm=vm,outmag=m,outvlam=vl,closest=closest)

sxaddpar,hdr,'PBNAME',name,' name of passband'
sxaddpar,hdr,'WAVEUNIT','Angstroms',' all wavelengths in Angstroms'
sxaddpar,hdr,'RESPUNIT',(pbergs ? 'ergs' : 'photons') $
  ,' energy (ergs) or photon sensitivity'
sxaddpar,hdr,'ZEROPT',zp,' passband zeropoint',format='F19.4'
sxaddpar,hdr,'VEGAFILE',vf,' Vega spectrum used in aux/standards/'
sxaddpar,hdr,'VEGAMAGS',vm,' set of Vega magnitudes in VEGA.mags.dat'
sxaddpar,hdr,'VEGAWEFF',vl,' Vega-weighted effective wavelength',format='F19.4'
sxaddpar,hdr,'MAGVEGA',m,' magnitude of Vega in this passband',format='F19.4'
sxaddhist,'Created using mkpassband.pro at UT ' + systime(/utc),hdr
sxaddhist,'Created by ' + getenv('USER') + '@' + getenv('HOST'),hdr

if addhist then sxaddhist,history,hdr

mwrfits,passband,outfn,hdr,ascii='D:G20.7',/create,/terminator
print,'done.'

end
