function amoebafit, sn, param0=param0, savefile=savefile, outdir=outdir

;; Commented on 2006/07/10 by Michael Wood-Vasey

;; This IDL routine uses the 'amoeba' fitter to find a local minimum
;; the supernova lightcurve fitter MLCS2k2 (Jha06).  The 'amoeba'
;; routine is built-in to IDL and uses the downhill simplex method.
;;
;; For reasons of efficiency, most of the information is passed
;; through the common blocks 'm2setup' and 'm2chisq'
;; ("m2" refers to the fitter name: "MLCS2k2").

common m2setup, c, s, n, sall, rfcname

;; The MLCS2k2 lightcurve model parameters and data to fit
;;   Some of the parameters are unused but we have to specify the
;;   m2chisq common block to contain all of them because other
;;   routines using this common block will use them.
;;  
;; fixarr  :  Set for those parameters that are fixed
;; fixval  :  The fixed values for those parameters that are fixed
;; vectors :  The MLCS2k2 lightcurve model vectors
;; redden  :  The MLCS2k2 reddening law (includes differential effect of phase, etc.)
;; info    :  Will be loaded with the information on the supernova being fitted
;; obspb   :  Not directly used here for the fit but stored in the
;; restpb  :    output structure.
;; data    :  Not used here
;; orpbind :       "
;; kctables:       "
;; gxtables:       "
;; cinv    :       "
;; npts    :  Number of points in the lightcurve.  Use to calculate
;;             the effective degrees of freedom of the fit
;; startparam : Gets set to the starting parameter values
;; nuniqf  :  Not used
common m2chisq, fixarr, fixval, vectors, redden, info, obspb, restpb, data, orpbind, $
  kctables, gxtables, cinv, npts, startparam, nuniqf
  
; we have 5 fit parameters
nparam = 5

info = readsninfo(sn)

np0 = n_elements(param0)
if np0 eq 0 then param0 = getlastparam(sn,info,outdir=outdir) $
else if np0 ne nparam then $
  message,' invalid array startparam'

param0 = checkfixedparam(sn,param0,fixarr=fixarr,fixval=fixval)
startparam = param0

; this does the setup work
setupfit, sn

; stick prior directory at the front of the path:
oldpath = !path
m2dir = getenv('MLCS2K2_BASEDIR') + '/'
priordir = m2dir + 'fit/priors'
!path = priordir + path_sep(/search_path) + !path


; non-linear parameters are minimized via amoeba
;  the scale array determines the size of the initial amoeba simplex,
;  i.e. a rough idea of the range of the parameters to search
;                t0    delta    rv     mu0     av0
amoebascale = [ 7.0d,  0.5d,   0.05d,  10.0d,  0.5d]
firstguess = startparam

message,'           firstguess: ' + $
    string(firstguess,format='(f10.3,f7.3,f7.3,f8.3,f7.3)'),/info

; update our starting point with a coarse amoeba
betterguess = amoeba(1d-2, function_name='getchisq', $
             p0=firstguess, scale=amoebascale, function_value=outchisq)

if n_elements(betterguess) ne 1 then begin  ; amoeba succeeded

    if outchisq[0] gt 1d11 then begin ; best fit in bad prior land
        ; so let's start over from the beginning
        betterguess = firstguess
        betterguess[1:4] = [0.3d, 3.1d, 35.0d, 0.2d]
    endif

    param1 = checkfixedparam(sn,betterguess,fixarr=fixarr,fixval=fixval)
    startparam = param1

    setupfit, sn

endif else begin   ; amoeba failed -- let's just press on
    betterguess = firstguess 
endelse
    
message,'          betterguess: ' + $
    string(betterguess,format='(f10.3,f7.3,f7.3,f8.3,f7.3)'),/info

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;     Actually do the fit      ;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
outp = amoeba(1d-5, function_name='getchisq', $
             p0=betterguess, scale=amoebascale)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

if n_elements(outp) eq 1 then begin  ; amoeba failed

    ntry = 1
    repeat begin
        ; tweak starting point by a tiny bit and try again
        betterguess[0:nparam-1] += randomn(seed,nparam)*amoebascale*0.03d*ntry*ntry

        message,'tweaking betterguess:' + $
          string(betterguess,format='(f10.3,f7.3,f7.3,f8.3,f7.3)'),/info

        outp = amoeba(1d-5, function_name='getchisq', $
                      p0=betterguess, scale=amoebascale)
        ntry++

    endrep until (n_elements(outp) ne 1 or ntry gt 5)
    
    if n_elements(outp) eq 1 then $
      message,'amoeba failed to converge 5 times.. giving up'
endif  

outp = fixparam(outp,fixarr,fixval)
message,'chisq and best fit parameters: ',/info
chisq = getchisq(outp,obslc=obslc,resid=resid,prisq=prisq,restphase=rphase, $
                      gxout=gx,kcout=kc,kcsig=kcsig,gxsig=gxsig,/print)

junk = where(fixarr eq 1, nfixed)
ndof = npts - nparam + nfixed

model = { obslc: obslc, $
          gx: gx, gxsig: sqrt(gxsig^2+(0.16d*gx)^2), $
          kc: kc, kcsig: kcsig, $
          c: c, s: s, n: n, $
          phase: rphase, $
          mingoodate: vectors.mingoodate, maxgoodate: vectors.maxgoodate }

out = { sn: sn, $
        info: info, $
        param: outp, $
        err: dblarr(nparam), $
        mincs: chisq, $
        csminparam: outp, $
        prisq: prisq, $
        ndof: ndof, $
        startparam: startparam, $
        obspb: obspb, $
        restpb: restpb, $
        data: data, $
        model: model, $
        resid: resid }

; switch back to old path
!path = oldpath

if n_elements(savefile) eq 1 then save,file=savefile,/compress

return, out

end
