function forcespec,lam,f,pbx,pby,color,guessrat=guessrat,tol=tol, $
                   maxiter=maxiter,outebv=outebv,status=status,errstop=errstop
;
; Version 2005-Nov-03
; 
; force a spectrum to have a particular color by warping it using
;   a Cardelli, Clayton \& Mathis R_V = 3.1 reddening law (with
;   O'Donnell update)
;
;     lam: wavelength
;       f: flux (f_lambda)
;     pbx: first passband in color (color is X-Y)
;     pby: second passband in color
;   color: value you want for the color
;
;   pbx and pby can be strings, like 'B' or 'V' or 'F775W_ACS' that
;   will be passed to getpb.pro, or if you've already called getbp.pro
;   they can be passband structures (faster for repeated calls)
;
;   guessrat: approximate guess of E(B-V)/E(X-Y) [default: 1.0]
;             this isn't critical, but play around with it if the
;             iteration doesn't converge
;             
;   tol:      tolerance for match [default: 0.001 mag]
;
;   maxiter:  maximum number of iterations [default: 100]
;             if we hit this limit (seems to only happen with
;             poorly sampled spectra), we set status = 1
;
;
;   outebv:   set this to a variable which will contain the amount
;             of ebv to unredden the spectrum by to get the desired
;             color, i.e.  ccm_unred,lam,f,outebv,outf would give 
;             outf the desired color
;
;   status:   output status indicator (0=success)
;
;   errstop:  set this to halt on any kind of error (e.g., maxiter)
;


if not keyword_set(guessrat) then guessrat = 1.0d
if not keyword_set(tol) then tol = 0.001d
if not keyword_set(maxiter) then maxiter = 100
errstop = keyword_set(errstop)

if size(pbx,/tname) eq 'STRING' then xpb = getpb(pbx) else xpb = pbx
if size(pby,/tname) eq 'STRING' then ypb = getpb(pby) else ypb = pby

status = 0

niter = 0
ebv = 0.0d
scolor = synphot(lam,f,xpb)-synphot(lam,f,ypb)

xx = [ebv]
yy = [scolor]

;; 2006/07/10 - MWV:  Where does the factor of 1.1 come from?
;; 2006/07/14 - sj: it's so we overshoot a little bit at first, so our
;;                  next guess is interpolating rather than extrapolating.
;;                  with the new algorithm, it doesn't much matter
ebv += 1.1d * guessrat * (scolor-color)
fastccm_unred,lam,f,ebv,out
scolor = synphot(lam,out,xpb)-synphot(lam,out,ypb)

; this converges much more quickly than before

; we create an array of tested ebv's and resulting colors
; and keep interpolating until we get close enough to the
; desired color
repeat begin

    xx = [xx,ebv]
    yy = [yy,scolor]
    sx = sort(xx)
    xx = xx[sx]
    yy = yy[sx]

    ebv = interpol(xx,yy,color)

    fastccm_unred,lam,f,ebv,out
    scolor = synphot(lam,out,xpb)-synphot(lam,out,ypb)
    ;;print,ebv,scolor

    niter++

endrep until ( (abs(scolor-color) le tol) or (niter gt maxiter) )

if niter ge maxiter then begin
    out = f
    status = 1
endif else begin
    outebv = ebv
endelse

if status ne 0 and errstop then $
  message,'not successful, status = ' + string(status)

return,out

end
