pro fullobslc, param, info, obspb, file, addobspb=addobspb $
             ,dates=dates,outpb=outpb,outrestpb=outrestpb $
             ,outlc=outlc,outerr=outerr $
             ,kcout=kcout,kcsig=kcsig,gxout=gxout,gxsig=gxsig $
             ,indates=indates,inphases=inphases

;; 2006/07/10 - Some additional commentary by Michael Wood-Vasey
;;   Calculates an MLCS2k2 model lightcurve from model parameters.
;;
;; Inputs:
;;
;;   param       : MLCS2k2 lightcurve model parameters
;;   info        : Information about the supernova being fit
;;   obspb       : Observed passbands
;;   file        : Filename for a output file with the generated lightcurve.  
;;                    If empty or undefined then no output file is generated.
;;
;; Optional Keywords
;;
;;   addobspb    : Additional observed passbands desired
;;   indates     : Dates on which to sample the light curve
;;   inphases    : Phases on which to sample the light curve 
;;                  [inphases is ignored/replaced if indates is supplied]
;;
;; Outputs
;;
;;   dates       : Dates for output lightcurve
;;   outpb       : Passbands for output lightcurve (generally same as
;;                   obspb but in possibly different order and doesn't
;;                   include passbands that didn't correspond to
;;                   the restframe UBVRI coverage of the MLCS2k2 model).
;;   outlc       : Output lightcurve
;;   outerr      : Total uncertainty in above (grey snakes: includes smatrix+kcorr+galext errors)
;;   kcout       : K-corrections used for output lightcurve
;;   kcsig       : Uncertainty in above
;;   gxout       : Milky Way galaxy extinction corrections used for output lightcurve
;;   gxsig       : Uncertainty in above

if n_elements(addobspb) eq 0 then nobspb = obspb $
else nobspb = [obspb,addobspb]

nobspb = nobspb[uniq(nobspb,sort(nobspb))]
nuniqf = n_elements(nobspb)

redden = getm2redden()
vectors = getm2vectors(redden.filters,info.vecname)
smatrix = getm2smatrix(info.vecname)

restpb = bestrest(nobspb, info.z, lameff=lameff)
s = sort(lameff)
nobspb = nobspb[s]
restpb = restpb[s]
lameff = lameff[s]

work = where(restpb ne 'none', nwork)

if nwork eq 0 then message,'no valid observer frame passbands!' $
else if nwork lt nuniqf then $
  message,'only valid passbands: ' + strjoin(nobspb[work],' '),/info

nobspb = nobspb[work]
restpb = restpb[work]
lameff = lameff[work]
nuniqf = nwork

rfcname = defaultcolor(restpb)

if n_elements(indates) ne 0 then inphases = (indates - param[0])/(1.0d + info.z)

if n_elements(inphases) ne 0 then begin

    qq = where(inphases ge vectors.mingoodate and inphases le vectors.maxgoodate,nqq)
    if nqq le 0 then message,'no valid dates/phases!'

    restphase = inphases[qq]
    dates = restphase*(1.0d + info.z) + param[0]
    ndate = n_elements(dates)

    rlc = dblarr(ndate,vectors.nfilters)
    colors = rlc

    for j=0,vectors.nfilters-1 do begin
        rlc[0,j] = getrestlc(param,vectors,redden,lonarr(ndate)+j,restphase,colors=tmpcol)
        colors[0,j] = tmpcol
    endfor


endif else begin    ; we don't have any input dates/phases

    restphase = vectors.date
    dates = restphase*(1.0d + info.z) + param[0]
    ndate = n_elements(dates)
    rlc = getrestlc(param,vectors,redden,colors=colors)

endelse

outpb = nobspb
outrestpb = restpb
outlc = dblarr(ndate,nuniqf)
outerr = outlc
kcout = outlc
kcsig = outlc
gxout = outlc
gxsig = outlc

defaultskcgx, rfcolset, gxlaw, gxrv

for i=0,nuniqf-1 do begin

    vind = where(vectors.filters eq restpb[i],nv)
    if nv ne 1 then message,'huh? something not right...'
    vind = vind[0]
    
    kcout[0,i] = getkc(restphase,colors[*,vind],rfcname[i],restpb[i], $
                       nobspb[i],info.z,sample=info.specsamp, $
                       rfcolset=rfcolset, sigma=ks)
    kcsig[0,i] = ks

    gxout[0,i] = getgx(info.ebvsfd,restphase,colors[*,vind],rfcname[i], $
                       nobspb[i],info.z,sample=info.specsamp, $
                       rfcolset=rfcolset,extlaw=gxlaw,rv=gxrv,sigma=gs)
    gxsig[0,i] = sqrt(gs^2 + (0.16d*reform(gxout[*,i]))^2)

    outlc[0,i] = reform(rlc[*,vind]) + kcout[*,i] + gxout[*,i]

    outerr[0,i] = sqrt( reform((gxsig[*,i])^2) + $
                        reform((kcsig[*,i])^2) + $
                        reform(interpol(smatrix.salldiag[*,vind],smatrix.date,restphase)) )

endfor


if n_elements(file) ne 0 then begin

    openw,lun,file,/get_lun

    begstr = '  '
    for q=0,22 do begstr = begstr + '#'
    printf,lun,'#######  MLCS2k2 fullobslc.pro  ' + systime() + begstr
    printf,lun,'#'
    printf,lun,'# output file: ' + file
    printf,lun,'#'
    printf,lun,'# input parameters: '
    printf,lun,'#   time of Bmax                       t0 = '+strtrim(param[0],2)
    printf,lun,'#   distance modulus [H0=65]         mu65 = '+strtrim(param[3],2)
    printf,lun,'#   delta                             del = '+strtrim(param[1],2)
    printf,lun,'#   V-band host extinction at tmax    av0 = '+strtrim(param[4],2)
    printf,lun,'#   extinction law                     rv = '+strtrim(param[2],2)
    printf,lun,'#   redshift                            z = '+strtrim(info.z,2)
    printf,lun,'#   MW SFD extinction              galebv = '+strtrim(info.ebvsfd,2)
    printf,lun,'#'
    printf,lun,'#   spectral library used        specsamp = '+strtrim(info.specsamp,2)
    printf,lun,'#   vectors used                  vecname = '+strtrim(info.vecname,2)
    printf,lun,'#'
    printf,lun,'# passbands   observer-frame    rest-frame'
    ostr = strarr(nuniqf)
    for i=0,nuniqf-1 do begin
        os = nobspb[i]
        while strlen(os) le 16 do os = os + ' '
        ostr[i] = os
        printf,lun,'#              ' + ostr[i] + '    ' + restpb[i]
    endfor
    printf,lun,'#'
    printf,lun,'# observer frame light curve and model uncertainites:'
    printf,lun,'#'
    printf,lun,'# passband                  t        mag    +/-     kcorr   +/-    galext  +/-'

    for i=0,nuniqf-1 do begin
        for j=0,ndate-1 do begin

            s = '     ' + ostr[i] + string(dates[j],format='(f11.3)') + $
                string(outlc[j,i],format='(f8.3)') + $
                string(outerr[j,i],format='(f7.3)') + $
                string(kcout[j,i],format='(f9.3)') + $
                string(kcsig[j,i],format='(f7.3)') + $
                string(gxout[j,i],format='(f8.3)') + $
                string(gxsig[j,i],format='(f7.3)')

            printf,lun,s

        endfor
    endfor

    close,lun
    free_lun,lun

endif

end
