pro fullrestlc, param, info, file $
             ,dates=dates,outpb=outpb $
             ,outlc=outlc,outerr=outerr $
             ,applymu=applymu
;;
;; Calculates model rest-frame light curve for given parameters
;;
;; Inputs:
;;
;;   param       : MLCS2k2 lightcurve model parameters
;;   info        : Information about the supernova being fit
;;   file        : Filename for a output file with the generated lightcurve.  
;;                    If empty or undefined then no output file is generated.
;; Keywords:
;;
;;   applymu     : If set we will add mu65 (param[3]) to the magnitudes.
;;                 By default, we return absolute magnitudes.

;; Outputs
;;
;;   dates       : Dates for output lightcurve
;;   outpb       : Passbands for output lightcurve
;;   outlc       : Output lightcurve
;;   outerr      : sqrt of the diagonal elements of the S matrix

applymu = keyword_set(applymu)

redden = getm2redden()
vectors = getm2vectors(redden.filters,info.vecname)
smatrix = getm2smatrix(info.vecname)

if total(vectors.filters eq smatrix.filters) ne vectors.nfilters then $
  message,'incompatible filter sets?'

calcparam = param
if not applymu then calcparam[3] = 0.0d
rlc = getrestlc(calcparam,vectors,redden,colors=colors)

dates = vectors.date
outpb = vectors.filters
outlc = rlc
outerr = sqrt(smatrix.salldiag)

if n_elements(file) ne 0 then begin

    openw,lun,file,/get_lun

    begstr = '  '
    for q=0,22 do begstr = begstr + '#'
    printf,lun,'#######  MLCS2k2 fullrestlc.pro  ' + systime() + begstr
    printf,lun,'#'
    printf,lun,'# output file: ' + file
    printf,lun,'#'
    printf,lun,'# input parameters: '
    printf,lun,'#   time of Bmax                       t0 = '+strtrim(param[0],2)+ $
      ' [NOT APPLIED]'
    printf,lun,'#   distance modulus [H0=65]         mu65 = '+strtrim(param[3],2)+ $
      (applymu ? ' ' : ' [NOT APPLIED]')
    printf,lun,'#   delta                             del = '+strtrim(param[1],2)
    printf,lun,'#   V-band host extinction at tmax    av0 = '+strtrim(param[4],2)
    printf,lun,'#   extinction law                     rv = '+strtrim(param[2],2)
    printf,lun,'#   redshift                            z = '+strtrim(info.z,2) + $
      ' [NOT USED]'
    printf,lun,'#'
    printf,lun,'#   vectors used                  vecname = '+strtrim(info.vecname,2)
    printf,lun,'#'
    printf,lun,'# rest-frame light curve and model uncertainites (sqrt(diag(S)):'
    printf,lun,'#'
    printf,lun,'# passband   phase     mag     +/-'

    for i=0,vectors.nfilters-1 do begin
        for j=0,vectors.ndate-1 do begin

            s = '     ' + outpb[i] + string(dates[j],format='(f13.3)') + $
                string(outlc[j,i],format='(f9.3)') + $
                string(outerr[j,i],format='(f7.3)')
            printf,lun,s

        endfor
    endfor

    close,lun
    free_lun,lun

endif

end
