function getgx,ebv,ph,rfcol,rfcolname,obsf,z $
              ,sample=sample,rfcolset=rfcolset,extlaw=extlaw,rv=rv $
              ,gx_lookup=gx_lookup $
              ,sigma=sigma
;
; getgx.pro -- computes Galactic extinction in a given passband
;
; Created 2005-Nov-14
; Version 2006-Oct-24
;               
; INPUTS
;       ebv: Galactic E(B-V)_true (scalar; from Schlegel, Finkbeiner, & Davis)
;        ph: rest-frame phases (scalar or array)
;     rfcol: rest-frame colors matching phases (scalar or array)
; rfcolname: rest-frame color name (string: 
;                                  'U-B', 'B-V', 'V-R', 'R-I', or 'V-I')
;      obsf: observer-frame passband (string or passband structure)
;         z: redshift (scalar)
;     
; RETURN VALUE
;  Galactic extinction [mag] in obsf passband (same number of elements as ph)
;
; OPTIONAL INPUTS
;    sample: spectral sample to use (string, default: 'normal')
;  rfcolset: rest-frame color set (string, default: 'Bessell')
;    extlaw: extinction law (string, 'CCM' [default] or 'FM')
;        rv: R_V (default: 3.1)
;
; gx_lookup: pre-loaded Galactic extinction lookup table structure;
;            if you supply this (from gx_loadfile), the inputs for
;            rfcolname, obsf, z, sample, rfcolset, extlaw, and rv are ignored
;
; OPTIONAL OUTPUTS
;     sigma: internal Galactic extinction uncertainty (same size as ph)
;
;        **NOTE** this is only the contribution to the uncertainty from the
;        spectral calculation. It does not include the uncertainty because
;        your input colors are uncertain, or the uncertainity on E(B-V)_true
;        itself (which is ~16% according to SFD; and of course correlated
;        over all the different passbands).
;       
;        You will almost definitely want to augment this uncertainty
;        to take those (dominant) effects into account.
;        

ngxl = n_elements(gx_lookup)
if ngxl gt 1 then message,'multiple gxorr lookup tables not supported'
if ngxl eq 0 then begin
    ; we need to look at the other arguments
    if n_elements(sample) eq 0 then sample = 'normal'
    if n_elements(rfcolset) eq 0 then rfcolset = 'Bessell'
    if n_elements(extlaw) eq 0 then extlaw='CCM'
    if n_elements(rv) eq 0 then rv=3.1
    if n_elements(z) ne 1 then $
      message,'redshift must be a scalar'
    if n_elements(rfcolname) ne 1 then $
      message,'multiple rest-frame colors not supported yet'
    loadtable = 1
endif else begin
    loadtable = 0
endelse

n = n_elements(ph)
if n_elements(rfcol) ne n then $
  message,'phases and rest-frame color array have different sizes'

if loadtable then begin
    ; first we determine the name of the lookup table
    fn = gx_getfilename(z,rfcolname,obsf,sample,rfcolset,extlaw,rv)
    ; load the lookup table
    gxtable = gx_loadfile(fn)
endif else begin
    gxtable = gx_lookup
endelse

; calculate the Galactic extinction and internal errors via interpolation
gx_lookup,gxtable,ph,ebv,rfcol,outgx,sigma

; return them
return,outgx

; calculate filter match #?
; calculate whether this is a good idea?
;    i.e. rest-frame color should constrain rest-frame filter

end
