function gridfit, sn, savefile=savefile, outdir=outdir

;; 2006/07/10 - Some additional commentary by Michael Wood-Vasey
;;     Do the MLCS2k2 lightcurve fit for a grid of input parameters
;;     Calculates the minimum chisq point within the grid as well as
;;     the 1D marginalizations for each fit parameter.


;; MLCS2k2 setup common block
;;
;; c        :  MLCS2k2 c matrix (c=s+n)
;; s        :  MLCS2k2 s matrix - MLCS2k2 model covariance matrix  
;;               interpolated from sall onto dates of observations
;; n        :  MLCS2k2 n matrix - Noise matrix for actual observations
;; sall     :  full s matrix (sampled every day for all filters)
;; rfcname  :  restframe filter color names for K- and MW-corrections
common m2setup, c,s,n,sall,rfcname

;; The MLCS2k2 lightcurve model parameters and data to fit
;;   Some of the parameters are unused but we have to specify the
;;   m2chisq common block to contain all of them because other
;;   routines using this common block will use them.
;;  
;; fixarr  :  Set for those parameters that are fixed
;; fixval  :  The fixed values for those parameters that are fixed
;; vectors :  The MLCS2k2 lightcurve model vectors
;; redden  :  The MLCS2k2 reddening law (includes differential effect of phase, etc.)
;; info    :  Will be loaded with the information on the supernova being fitted
;; obspb   :  Observed passbands
;; restpb  :  Restframe passbands
;; data    :  Observe data
;; orpbind :  observed->restframe passband index
;; kctables:  loaded K-correction
;; gxtables:  loaded MW extinction-correction
;; cinv    :  Inverse of the C matrix
;; npts    :  Number of points in the lightcurve.  Use to calculate
;;             the effective degrees of freedom of the fit
;; startparam : Gets set to the starting parameter values
;; nuniqf  :  Not used
common m2grid, grid,t0,del,rv,mu0,av0,nt0,ndel,nrv,nmu0,nav0 $
  ,dt0,ddel,drv,dmu0,dav0,csgrid,finalnpoints

common m2chisq, fixarr,fixval,vectors,redden,info,obspb,restpb,data,orpbind $
  ,kctables,gxtables,cinv,npts,startparam,nuniqf
 
; we have 5 fit parameters
nparam = 5

; the grid takes care of what's fixed or not
fixarr = bytarr(nparam)             ;; Boolean array denoting whether or not a parameter is fixed for the fit
fixval = dblarr(nparam) - 999.999d  ;; Set default to extreme value to catch later

info = readsninfo(sn)               ;; Read in the information about the supernova 

startparam = getlastparam(sn,info,lastiter=lastiter,err=startperr,outdir=outdir)
if lastiter lt 0 then $
  message,'probably not wise to run gridfit before running amoebafit once',/info

; this sets up the grid
setupgrid, sn, startparam, startperr

; this does the setup work for the fit
setupfit, sn

; stick prior directory at the front of the path:
oldpath = !path
m2dir = getenv('MLCS2K2_BASEDIR') + '/'
priordir = m2dir + 'fit/priors'
!path = priordir + path_sep(/search_path) + !path

junk = where(finalnpoints eq 1, nfixed)
ndof = npts - nparam + nfixed

now = dblarr(5)

nn = 0L
message,'grid size: ' + string(n_elements(csgrid)),/info
print,'starting t0 del rv mu0 av0       ',grid.centers,format='(a0,5f9.3)'

for it0=0L,nt0-1 do begin
    now[0] = t0[it0]

    for idel=0L,ndel-1 do begin
        now[1] = del[idel]

        for irv=0L,nrv-1 do begin
            now[2] = rv[irv]

            for imu0=0L,nmu0-1 do begin
                now[3] = mu0[imu0]

                for iav0=0L,nav0-1 do begin
                    now[4] = av0[iav0]

                    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    ;;;;; Actually calculate the chisq ;;;;;
                    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;                    
                    chisq = getchisq(now)
                    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    nn = nn+1

                    ;; Store this chisq value in our grid
                    csgrid[it0,idel,irv,imu0,iav0] = chisq

                    if nn mod 2000L eq 0 then print,nn,ndof,chisq,now, $
                        format='(i9,i6,f16.5,f11.3,4f9.3)'

                endfor
            endfor
        endfor
    endfor
endfor


; switch back to old path
!path = oldpath

mincs = min(csgrid,minind)
ind = array_indices(csgrid,minind)
csmin = [t0[ind[0]],del[ind[1]],rv[ind[2]],mu0[ind[3]],av0[ind[4]]]

test = getchisq(csmin,prisq=prisq)
if test ne mincs then message,'huh? something changed!'

;; Transform from chisq to probability space
pgrid = exp((mincs-csgrid)/2.0d)
pgrid = pgrid/total(pgrid*dt0*ddel*drv*dmu0*dav0)

;; Calculate 1D marginalized probabilities for each fit parameter
pt0 = dblarr(nt0)
for z=0,nt0-1 do pt0[z] = total(pgrid[z,*,*,*,*]*ddel*drv*dmu0*dav0)

pdel = dblarr(ndel)
for z=0,ndel-1 do pdel[z] = total(pgrid[*,z,*,*,*]*dt0*drv*dmu0*dav0)

prv = dblarr(nrv)
for z=0,nrv-1 do prv[z] = total(pgrid[*,*,z,*,*]*dt0*ddel*dmu0*dav0)

pmu0 = dblarr(nmu0)
for z=0,nmu0-1 do pmu0[z] = total(pgrid[*,*,*,z,*]*dt0*ddel*drv*dav0)

pav0 = dblarr(nav0)
for z=0,nav0-1 do pav0[z] = total(pgrid[*,*,*,*,z]*dt0*ddel*drv*dmu0)

;; Calculate means for each fit parameter
;;    the minimum mu error has to be 0.001 so we know a grid fit was done
mu0mean = total(mu0*pmu0*dmu0)
mu0sd = sqrt((total(mu0*mu0*pmu0*dmu0) - mu0mean*mu0mean) > 1d-6)


m0mean = mu0mean + getzeropoint(info.vecname)
m0sd = mu0sd

av0mean = total(av0*pav0*dav0)
av0sd = sqrt((total(av0*av0*pav0*dav0) - av0mean*av0mean) > 0.0d)

delmean = total(del*pdel*ddel)
delsd = sqrt((total(del*del*pdel*ddel) - delmean*delmean) > 0.0d)

t0mean = total(t0*pt0*dt0)
t0sd = sqrt((total(t0*t0*pt0*dt0) - t0mean*t0mean) > 0.0d)

rvmean = total(rv*prv*drv)
rvsd = sqrt((total(rv*rv*prv*drv) - rvmean*rvmean) > 0.0d)

outp = [t0mean, delmean, rvmean, mu0mean, av0mean]
outperr = [t0sd, delsd, rvsd, mu0sd, av0sd]

print,'mean parameters: ' + string(outp,format='(f12.3,4f9.3)')

csp = getchisq(outp,resid=cspresid,prisq=cspprisq,obslc=cspolc,restphase=rphase, $
              gxout=cspgx,kcout=cspkc,kcsig=cspkcsig,gxsig=cspgxsig)

model = { obslc: cspolc, $
          gx: cspgx, gxsig: sqrt(cspgxsig^2+(0.16d*cspgx)^2), $
          kc: cspkc, kcsig: cspkcsig, $
          c: c, s: s, n: n, $
          phase: rphase, $
          mingoodate: vectors.mingoodate, maxgoodate: vectors.maxgoodate }

out = { sn: sn, $
        info: info, $
        param: outp, $
        err: outperr, $
        mincs: mincs, $
        csminparam: csmin, $
        prisq: prisq, $
        ndof: ndof, $
        startparam: startparam, $
        obspb: obspb, $
        restpb: restpb, $
        data: data, $
        model: model, $
        resid: cspresid }

if n_elements(savefile) eq 1 then save,file=savefile,/compress

return,out

end
