pro gx_createfile,fn,silent=silent
;
; gx_createfile.pro -- creates the Galactic extinction lookup table
;     this is the routine that really does the work
;
; Created 2005-Nov-14
; Version 2006-Nov-01
;               
; INPUTS
;        fn: lookup table file name, e.g.,
;           aux/galext/normal/gx.z=0.032.R_Bessell.V-R.Bessell.CCM.R_V=3.1.fits
;
; KEYWORDS:
;    silent: if set we don't print out informational messages

silent = keyword_set(silent)

m2dir = getenv('MLCS2K2_BASEDIR') + '/'
outfn = m2dir+fn

; make sure it's not already there
if file_test(outfn) then message,' file already exists: ' + fn

; break the filename into its parts
gx_parsefilename,fn,z,rfcolname,obsf,sample,rfcolset,extlaw,rv
z1 = 1.0d + z

; set modelspec if spectra are based on a smooth model
;   as opposed to being real, noisy data. for now, only 
;   the nugent* samples are model spectra
modelspec = (strmid(sample,0,6) eq 'nugent') ? 1 : 0

if not silent then $
  message,'creating Galactic extinction lookup table: ' + fn,/info

opb = getpb(obsf)
oxmin = min(opb.wavelength)
oxmax = max(opb.wavelength)

; we first need to get the information to colorshift the spectra
csdir = m2dir + 'aux/colorshift/results/' + sample + '/'

infofn = 'INFO.' + rfcolset
spcsav = '' & spccolname= '' & spcrestset = ''
readit,csdir+infofn,spcsav,spcph,spcxmin,spcxmax,spcnc,spccolname,spcrestset 

; now find the spectra that can calculate this combination
;   they need to be the right color and colorset
;   they need to cover the observer-frame passband at redshift z
qq = where(spcrestset eq rfcolset and spccolname eq rfcolname and $
           spcxmin*z1 le oxmin and spcxmax*z1 ge oxmax $
           ,nqq)
if nqq le 0 then message,'no matching spectra in '+csdir+infofn

;; sort by spectral phase because we're going to interpolate over phase later 
n = nqq
qq = qq[sort(spcph[qq])]
ssav = spcsav[qq]
sph = spcph[qq]
sxmin = spcxmin[qq]
sxmax = spcxmax[qq]
snc = spcnc[qq]

if not silent then $
  message,'  calculating with ' + string(n,format='(i0)') + ' spectra...',/info

if min(snc) ne max(snc) then message,'something is wrong...'

restore,csdir+ssav[0]
colarr = color
ncol = ncolor


; The Galactic extinction is well-described by a quadratic in ebv
; for any particular rest-frame color. This is equivalent to saying
; the R value is linear in ebv, so we store our lookup tables as 
; two numbers, and intercept and a slope, such that
; R = intercept + slope*ebv (and then Galext = R*ebv).
;
; We could have stored two "images" of intercepts and slopes for 
; each value of the phase and rest-frame color, but it turns out
; both of these are also well-fit as polynomial functions of the
; color. So for each phase we store 9 coefficients, 4 for the
; intercept and 5 for the slope. 

; we need to calculate the slope and intercept, 3 points works fine.
; it's been tested with a lot more, but this saves a lot of time.
ebv = [0.02,0.05,1.0]
nebv = n_elements(ebv)

xgx = dblarr(n,ncol,nebv)

ai = dblarr(n,ncol)
as = dblarr(n,ncol)

; calculate the extinction for every spectrum
;
for i=0,n-1 do begin

    restore,csdir+ssav[i]
    if (ncolor ne ncol) or (max(abs(color-colarr)) ne 0d) then $
      message,'invalid file: '+csdir+ssav[i]

    lamz = lam*z1
    
    ; make the spectrum have the desired rest-frame color
    ;  this is just the colorshifting it has to be ccm_unred
    fastccm_unred,lam,f,ebvarr,cshifted

    if extlaw eq 'CCM' then begin
        fastccm_unred,lamz,dblarr(n_elements(f))+1d,-ebv,attn
    endif

    for j=0,ncol-1 do begin

        ; calculate the unextinguished observer-frame magnitude
        m0 = synphot(lamz,cshifted[*,j],opb)

        for k=0,nebv-1 do begin

            if extlaw eq 'CCM' then begin
                nout = cshifted[*,j]*attn[*,k]
            endif else if extlaw eq 'FM' then begin
                fm_unred,lamz,cshifted[*,j],-ebv[k],nout,r_v=rv
            endif else begin
                message,'unknown extinction law: ' + extlaw
            endelse
            
            xgx[i,j,k] = (synphot(lamz,nout,opb)-m0)/ebv[k]

        endfor

        out = linfit(ebv,xgx[i,j,*])
        ai[i,j] = out[0]
        as[i,j] = out[1]

    endfor

endfor

; interpolate & smooth the results onto a uniform phase array -25d to +125d
;   we also need to calculate the RMS as a function of color
pharr = vec(-25d,125d,1.0d,nelems=nph)

outai = dblarr(nph,ncol)
outisig = dblarr(ncol)

outas = outai
outssig = outisig

; our smoothing length is ~20 days, so we have about 6 degrees of
; freedom over the whole range
; but don't smooth if we are using model spectra

sfilt = (modelspec) ? [1d] : savgol(9,9,0,3)
ndof = 6.0d



; first we do the intercept
; outlier rejection
;    no rejection in model spectra
rejsig = (modelspec) ? 1000d : 5d
maxpasses = (modelspec) ? 1 : 3
itossed = lonarr(ncol)

for j=0,ncol-1 do begin

    nowai = reform(ai[*,j])
    nowph = sph
    keep = lindgen(n)

    npass = 0
    repeat begin

        npass++

        nowph = nowph[keep]
        nowai = nowai[keep]
        nstart = n_elements(nowph)

        ; if we have more than one point at the same phase, average them
        nuph = nowph[uniq(nowph)]
        nuai = makeuniq(nowai,nowph)
        nnu = n_elements(nuph)

        ; add endpoints at "infinity" so that extrapolations are
        ;    just set to the endpoint value
        nuph = [-1d3,nuph,1d3]
        nuai = [nuai[0],nuai,nuai[nnu-1]]
        
        ; interpolate onto our phase array and smooth
        out = interpol(nuai,nuph,pharr)
        out = convol(out,sfilt,/edge_truncate)
        
        ; calculate residuals and rms
        resid = nowai - interpol(out,pharr,nowph)
        sigden = (nstart-ndof) > 1
        ssig = sqrt(total(resid^2)/sigden) + 1d-3
        
        keep = where(abs(resid) lt rejsig*ssig,nkeep)

    endrep until ( (nstart eq nkeep) or (npass gt maxpasses) )

    outai[0,j] = out
    outisig[j] = ssig
    itossed[j] = n - nstart

endfor


; now we do the slope

; outlier rejection
;    no rejection in model spectra
rejsig = (modelspec) ? 1000d : 5d
maxpasses = (modelspec) ? 1 : 3
stossed = lonarr(ncol)

for j=0,ncol-1 do begin

    nowas = reform(as[*,j])
    nowph = sph
    keep = lindgen(n)

    npass = 0
    repeat begin

        npass++

        nowph = nowph[keep]
        nowas = nowas[keep]
        nstart = n_elements(nowph)

        ; if we have more than one point at the same phase, average them
        nuph = nowph[uniq(nowph)]
        nuas = makeuniq(nowas,nowph)
        nnu = n_elements(nuph)

        ; add endpoints at "infinity" so that extrapolations are
        ;    just set to the endpoint value
        nuph = [-1d3,nuph,1d3]
        nuas = [nuas[0],nuas,nuas[nnu-1]]
        
        ; interpolate onto our phase array and smooth
        out = interpol(nuas,nuph,pharr)
        out = convol(out,sfilt,/edge_truncate)
        
        ; calculate residuals and rms
        resid = nowas - interpol(out,pharr,nowph)
        sigden = (nstart-ndof) > 1
        ssig = sqrt(total(resid^2)/sigden) + 1d-3
        
        keep = where(abs(resid) lt rejsig*ssig,nkeep)

    endrep until ( (nstart eq nkeep) or (npass gt maxpasses) )

    outas[0,j] = out
    outssig[j] = ssig
    stossed[j] = n - nstart

endfor

avgtossed = cmp_avg(double([itossed,stossed]))


; okay now we fit the intercept and slope as polynomial
;  functions of the rest-frame color to save space

icoeffs = dblarr(4,nph)
scoeffs = dblarr(5,nph)

fitisig = dblarr(nph)
fitssig = dblarr(nph)

for i=0,nph-1 do begin

    resi = reform(poly_fit(color,outai[i,*],3,yerr=resierr,/double))
    icoeffs[0,i] = resi
    fitisig[i] = resierr

    ress = reform(poly_fit(color,outas[i,*],4,yerr=resserr,/double))
    scoeffs[0,i] = ress
    fitssig[i] = resserr

endfor

finisig = sqrt((cmp_avg(outisig))^2 + (cmp_avg(fitisig))^2)
finssig = sqrt((cmp_avg(outssig))^2 + (cmp_avg(fitssig))^2)


if not silent then $
  message,'    on average, we rejected ' + $
          string(avgtossed/n*100.0d,format='(f0.2)') + $
          '% of them',/info

; save it as a fits file
gx_writefits,outfn,pharr,nph,colarr,icoeffs,scoeffs,finisig,finssig $
            ,z,rfcolname,obsf,sample,rfcolset,extlaw,rv $
            ,n,avgtossed

if not silent then $
  message,'done. wrote ' + outfn,/info

end
