pro kc_createfile,fn,silent=silent
;
; kc_createfile.pro -- creates the K-correction lookup table
;     this is the routine that really does the work
;
; Created 2005-Nov-09
; Version 2006-Nov-01
;               
; INPUTS
;        fn: lookup table file name 
;            (e.g., aux/kcorr/normal/kc.z=0.123.B_Bessell.to.B_Bessell.B-V.Bessell.fits)
;
; KEYWORDS:
;    silent: if set we don't print out informational messages

silent = keyword_set(silent)

m2dir = getenv('MLCS2K2_BASEDIR') + '/'
outfn = m2dir+fn

; make sure it's not already there
if file_test(outfn) then message,' file already exists: ' + fn

; break the filename into its parts
kc_parsefilename,fn,z,rfcolname,rf,obsf,sample,rfcolset
z1 = 1.0d + z

; set modelspec if spectra are based on a smooth model
;   as opposed to being real, noisy data. for now, only 
;   the nugent* samples are model spectra
modelspec = (strmid(sample,0,6) eq 'nugent') ? 1 : 0

if not silent then $
  message,'creating K-correction lookup table: ' + fn,/info

rpb = getpb(rf)
rxmin = min(rpb.wavelength)
rxmax = max(rpb.wavelength)

opb = getpb(obsf)
oxmin = min(opb.wavelength)
oxmax = max(opb.wavelength)


addsig = 0.0d

; kludge for template K-correction error (as opposed to based on real spectra)
if modelspec then begin
    ; placeholder -- make an error estimate based on the filter mismatch
    addsig += 0.01d
endif

; kludge for the far-U band where nothing is very good
if strmid(rpb.name,0,2) eq 'U_' then begin
    
    efflam = synflux(opb.wavelength,opb.wavelength,opb)
    reff = efflam/z1

    if reff lt 2500 then message, 'wow that filter is really blue! reff: '+string(reff),/info
    if reff lt 3500 then addsig += 0.6d * ((3500d - reff)/1000d)

endif
    

; we first need to get the information to colorshift the spectra
csdir = m2dir + 'aux/colorshift/results/' + sample + '/'

infofn = 'INFO.' + rfcolset
spcsav = '' & spccolname= '' & spcrestset = ''
readit,csdir+infofn,spcsav,spcph,spcxmin,spcxmax,spcnc,spccolname,spcrestset 

; now find the spectra that can calculate this K-correction combination
;   they need to be the right color and colorset
;   they need to cover the rest-frame passband at zero redshift
;   they need to cover the observer-frame passband at redshift z
qq = where(spcrestset eq rfcolset and spccolname eq rfcolname and $
           spcxmin le rxmin and spcxmax ge rxmax and $
           spcxmin*z1 le oxmin and spcxmax*z1 ge oxmax $
           ,nqq)
if nqq le 0 then message,'no matching spectra in '+csdir+infofn

n = nqq
qq = qq[sort(spcph[qq])]
ssav = spcsav[qq]
sph = spcph[qq]
sxmin = spcxmin[qq]
sxmax = spcxmax[qq]
snc = spcnc[qq]

if not silent then $
  message,'  calculating with ' + string(n,format='(i0)') + ' spectra...',/info

if min(snc) ne max(snc) then message,'something is wrong...'

restore,csdir+ssav[0]
colarr = color
ncol = ncolor

xkc = dblarr(n,ncol)

; the constant part
kz = 2.5d * alog10(z1)

; cross filter K-correction, see Kim, Goobar, & Perlmutter (1996)
; k_ro = kz + 2.5 log10(synflux(x,f,r)) - 2.5 log10(synflux(x*(1+z),f,o) - r.zp + o.zp
;      = kz - (r.zp - 2.5log10(synflux(x,f,r))) + (o.zp - 2.5log10(synflux(x*(1+z),f,o)))
;      = kz -        synphot(x,f,r)             +       synphot(x*(1+z),f,o) 
;      = kz -            mag0                   +              magz

; calculate it for every spectrum
for i=0,n-1 do begin

    restore,csdir+ssav[i]
    if (ncolor ne ncol) or (max(abs(color-colarr)) ne 0d) then $
      message,'invalid file: '+csdir+ssav[i]

    ; make the spectrum have the desired rest-frame color
    fastccm_unred,lam,f,ebvarr,out

    for j=0,ncol-1 do begin

        mag0 = synphot(lam,out[*,j],rpb)
        magz = synphot(lam*z1,out[*,j],opb)

        xkc[i,j] = kz - mag0 + magz

    endfor

endfor

; interpolate & smooth the results onto a uniform phase array -25d to +125d
;   we also need to calculate the RMS as a function of color
pharr = vec(-25d,125d,1.0d,nelems=nph)
outkc = dblarr(nph,ncol)
outksig = dblarr(ncol)

if not modelspec then begin
    ; our smoothing length is ~20 days, so we have about 6 degrees of
    ; freedom over the whole range
    sfilt = savgol(9,9,0,3)
    ndof = 6.0d

    ; outlier rejection
    rejsig = 3.5
    maxpasses = 3
endif else begin
    ; no smoothing/rejection with model spectra
    sfilt = [1d]
    ndof = 6d
    rejsig = 1000d
    maxpasses = 1
endelse

tossed = lonarr(ncol)

for j=0,ncol-1 do begin

    nowk = reform(xkc[*,j])
    nowph = sph
    keep = lindgen(n)

    npass = 0
    repeat begin

        npass++

        nowph = nowph[keep]
        nowk = nowk[keep]
        nstart = n_elements(nowph)

        ; if we have more than one point at the same phase, average them
        nuph = nowph[uniq(nowph)]
        nuk = makeuniq(nowk,nowph)
        nnu = n_elements(nuph)

        ; add endpoints at "infinity" so that extrapolations are
        ;    just set to the endpoint value
        nuph = [-1d3,nuph,1d3]
        nuk = [nuk[0],nuk,nuk[nnu-1]]
        
        ; interpolate onto our phase array and smooth
        out = interpol(nuk,nuph,pharr)
        out = convol(out,sfilt,/edge_truncate)
        
        ; calculate residuals and rms
        resid = nowk - interpol(out,pharr,nowph)
        sigden = (nstart-ndof) > 1
        ssig = sqrt(total(resid^2)/sigden) + 1d-3
        
        keep = where(abs(resid) lt rejsig*ssig,nkeep)

    endrep until ( (nstart eq nkeep) or (npass gt maxpasses) )

    outkc[0,j] = out
    outksig[j] = ssig
    tossed[j] = n - nstart

endfor

avgtossed = cmp_avg(double(tossed))

if not silent then $
  message,'    on average, we rejected ' + $
          string(avgtossed/n*100.0d,format='(f0.2)') + $
          '% of them',/info

; add in any additional error in the K-correction
outksig = sqrt(outksig^2 + addsig^2)


; save it as a fits file
kc_writefits,outfn,pharr,nph,colarr,ncol,outkc,outksig $
            ,z,rfcolname,rf,obsf,sample,rfcolset $
            ,n,avgtossed

if not silent then $
  message,'done. wrote ' + outfn,/info

end
