function synflux,x,spc,pb,plot=plot,oplot=oplot,allowneg=allowneg

;
; version 2006-May-30
;
;  do synthetic photometry on a spectrum
;
;  x:   wavelength points for spectrum  
;  spc: spectrum (f_lambda)
;  pb:  passband (from getpb.pro)
;
;  pb can also be a string, like 'B' or 'V' or 'F775W_ACS' that
;  will just be passed to getpb.pro, but you should do this
;  yourself if you are repeatedly calling synphot or synflux
;  because it's much faster to read in the passband once, so:
;
;  faster:                             also works, but slower:
;  b = getpb('B')                      print,synphot(x,f,'B')
;  print,synphot(x,f,b)                print,synphot(x2,f2,'B')
;  print,synphot(x2,f2,b) etc.         etc.
;
;  the faster method will also let you specify desired options to getpb.pro
;
;  KEYWORDS
;  allowneg: allow the passband to be negative (it's nonnegative by default)
;
;  plot: set to see a plot
;  oplot: just overplot the passband (good for repeated calls)
;
;  returns flux
;

if size(pb,/tname) eq 'STRING' then pbs = getpb(pb) else pbs = pb

nx = n_elements(x)
if (n_elements(spc) ne nx) then begin
    message,'x and spc have different sizes'
endif

if pbs.respunit eq 'photons' then begin
    pbphot = 1
endif else if pbs.respunit eq 'ergs' then begin
    pbphot = 0
endif else begin
    message,'unknown response unit in passband: ' + pbs.respunit
endelse

pbx = pbs.wavelength
pby = pbs.response

np = n_elements(pbx)
if (n_elements(pby) ne np) then begin
    message,' pbs.wavelength and pbs.response have different sizes'
endif

if nx eq 1 or np eq 1 then begin
    message,'warning! 1-element array passed, returning 0',/info
    return,spc[0]-spc[0]
endif

diffx = x[1:nx-1]-x[0:nx-2]
diffp = pbx[1:np-1]-pbx[0:np-2]

if (min(diffx) le 0) or (min(diffp) le 0) then begin
    message,'passed non-increasing wavelength array'
endif

if x[0] gt pbx[0] then begin
    message,'spectrum doesn''t go blue enough for passband!'
endif

if x[nx-1] lt pbx[np-1] then begin
    message,'spectrum doesn''t go red enough for passband!'
endif

g = where(x ge pbx[0] and x le pbx[np-1])  ; overlap range

pbspl = spl_interp(pbx,pby,pbs.splinit,x[g])  ; do it

if not keyword_set(allowneg) then pbspl = pbspl > 0

if keyword_set(plot) then plot,x,spc
if keyword_set(oplot) or keyword_set(plot) then $
  oplot,x[g],!y.crange[0]+pbspl/max(pbspl)*(!y.crange[1]-!y.crange[0])*0.9d,linestyle=2

; if the response is photon sensitivity we need to integrate
;    f_lam*response*lambda*dlambda
; otherwise we integrate f_lam*response*dlambda
if (pbphot) then pbspl *= x[g]

res = trapint(x[g],pbspl*spc[g])/trapint(x[g],pbspl)

return, res

end
